@@ -9,7 +9,13 @@ use warnings;
 
 use parent 'DBIx::Class::ResultSet';
 
-our $VERSION = 'v0.1.0';
+use Ref::Util qw/ is_plain_arrayref is_plain_hashref is_plain_scalarref /;
+
+# RECOMMEND PREREQ: Ref::Util::XS
+
+use namespace::clean;
+
+our $VERSION = 'v0.2.0';
 
 
 sub _resolved_attrs {
@@ -24,12 +30,12 @@ sub _resolved_attrs {
         my $from = $attrs->{from};
 
         $rs->throw_exception('tablesample on joins is not supported')
-            if (ref $from eq 'ARRAY') && @$from > 1;
+            if is_plain_arrayref($from) && @$from > 1;
 
-        $conf = { fraction => $conf } unless ref $conf;
+        $conf = { fraction => $conf } unless is_plain_hashref($conf);
 
         $rs->throw_exception('tablesample must be a hashref')
-          unless ref $conf eq 'HASH';
+            unless is_plain_hashref($conf);
 
         my $sqla = $rs->result_source->storage->sql_maker;
 
@@ -40,13 +46,16 @@ sub _resolved_attrs {
         }
 
         my $arg = $conf->{fraction};
+        $arg = $$arg if is_plain_scalarref($arg);
         $part_sql .= "($arg)";
 
         if ( defined $conf->{repeatable} ) {
-            $part_sql .= sprintf( ' repeatable (%s)', $conf->{repeatable} );
+            my $seed = $conf->{repeatable};
+            $seed = $$seed if is_plain_scalarref($seed);
+            $part_sql .= sprintf( ' repeatable (%s)', $seed );
         }
 
-        if (ref $from eq 'ARRAY') {
+        if (is_plain_arrayref($from)) {
             my $sql = $sqla->_from_chunk_to_sql($from->[0]) . $sqla->_sqlcase($part_sql);
             $from->[0] = \$sql;
         }
@@ -75,7 +84,7 @@ DBIx::Class::Helper::TableSample - Add support for tablesample clauses
 
 =head1 VERSION
 
-version v0.1.0
+version v0.2.0
 
 =head1 SYNOPSIS
 
@@ -118,14 +127,29 @@ reference:
 
 =item C<fraction>
 
-This is the percentage or fraction of the table to sample.
+This is the percentage or fraction of the table to sample,
+between 0 and 100, or a numeric expression that returns
+such a value.
 
-Depending on your database, this can be a decimal or must
-be an integer.
+(Some databases may restrict this to an integer.)
 
 The value is not checked by this helper, so you can use
 database-specific extensions, e.g. C<1000 ROWS> or C<15 PERCENT>.
 
+Scalar references are dereferenced, and expressions or
+database-specific extensions should be specified has scalar
+references, e.g.
+
+  my $rs = $schema->resultset('Wobbles')->search_rs(
+    undef,
+    {
+      columns     => [qw/ id name /],
+      tablesample => {
+        fraction => \ "1000 ROWS",
+      },
+    }
+  );
+
 =item C<type>
 
 By default, there is no sampling type., e.g. you can simply use:
@@ -155,6 +179,8 @@ to generate
 If your database supports or requires a type, you can specify it,
 e.g. C<system> or C<bernoulli>.
 
+See your database documentation for the allowable types.
+
 =item C<repeatable>
 
 If this key is specified, then it will add a REPEATABLE clause,
@@ -175,6 +201,10 @@ to generate
 
   SELECT me.id FROM artist me TABLESAMPLE (5) REPEATABLE (123456)
 
+Scalar references are dereferenced, and expressions or
+database-specific extensions should be specified has scalar
+references.
+
 =back
 
 Resultsets with joins or inner queries are not supported.
@@ -183,8 +213,9 @@ Resultsets with joins or inner queries are not supported.
 
 This module is experimental.
 
-Not all databases support table sampling, they may have different
-restrictions.  You should consult your database documentation.
+Not all databases support table sampling, and thoser that do may have
+different restrictions.  You should consult your database
+documentation.
 
 =head1 SEE ALSO
 
